import os
import argparse
from word2wiz import word2wiz
from .version import __version__


def isdoc(filepath):
    f = filepath.lower()
    return f.endswith('.docx') or f.endswith('.doc')


def get_files(args):
    files = []
    for input_file in args.input_files:
        if os.path.isfile(input_file):
            files += [input_file]
        elif os.path.isdir(input_file):
            root, dirs, dir_files = next(os.walk(input_file))
            files += [os.path.join(root, f) for f in dir_files if isdoc(f)]
    return files


def convert_single(path, args):
    """
    Run the tool for one single file
    """
    spell, report = word2wiz.word2wiz(path)

    if args.output:
        # Write the spell
        with open(args.output, 'w') as output_file:
            output_file.write(spell)
    else:
        # By default, print the spell on the stdout
        print(spell)

    if args.report:
        # Write the report
        with open(args.report, 'w') as report_file:
            report_file.write(report)
    elif args.output:
        # Only print the report if the spell hasn't been printed
        print(report)


def convert_batch(paths, args):
    """
    Run the tool for multiple files
    """
    for path in paths:
        print('Converting {0}'.format(path))
        spell, report = word2wiz.word2wiz(path)

        path_base, _ = os.path.splitext(path)
        filename = os.path.basename(path_base)
        if args.output:
            spell_path = os.path.join(args.output, filename + '.spl')
        else:
            spell_path = path_base + '.spl'

        if args.report:
            report_path = os.path.join(args.report, filename + '.txt')
        else:
            report_path = path_base + '.txt'

        # Write the spell
        with open(spell_path, 'w') as output_file:
            output_file.write(spell)

        # Write the report
        with open(report_path, 'w') as report_file:
            report_file.write(report)


def main():
    description = 'Create a spell configuration from a Microsoft Word document'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('input_files',
                        metavar='FILES',
                        nargs='+',
                        help='the input word document')

    parser.add_argument('-o', '--output',
                        help='the file for the output spell (stdout by default)'
                        )

    parser.add_argument('--report',
                        help='the ouput report file containing the fields and' +
                        'the metadatas they are linked to')

    parser.add_argument('-v', '--version',
                        help='prints the version',
                        action='version',
                        version='%(prog)s ' + __version__)

    # Parse
    args = parser.parse_args()

    # Files to convert
    files = get_files(args)

    # Run
    if len(files) == 1:
        convert_single(files[0], args)
    else:
        convert_batch(files, args)



#!/usr/bin/python

import re
from docx import Document


def analyse_doc(docx_file):
    document = Document(docx_file)

    marks = [
        ('«', '»'),
        ('<<', '>>')
    ]

    # Regex parts for the delimiters
    start_dels, end_dels = ('|'.join(m[i] for m in marks)
                            for i in [0, 1])
    chars_with_repetitions = ''.join([m[0]+m[1] for m in marks])
    chars = ''.join(list(set(chars_with_repetitions)))
    # Regex to find the questions
    regex = '(?:{0})([^{1}]*)(?:{2})'.format(start_dels, chars, end_dels)

    questions = []
    for p in document.paragraphs:
        questions += re.findall(regex, p.text)
    return questions



import re
from os.path import dirname, realpath, join
from .spell_helper import Step, LabelControl, make_control
from .util import parse_mark

DEFAULT_STEP_NAME = 'doc_name'
DEFAULT_STEP_GROUP_NAME = 'Buitenland'


def remove_unwanted_matches(marks, file_path=None):
    """
    Loads the unwanted matches from unwanted_matches.txt (by default), and
    returns a list with the marks that do not match.
    Args:
        marks(list): A list of strings with all the matches
        file_path(str): Optional. The file that contains the unwanted matches.
    Returns:
        The input list minus the marks that are in the unwanted_matches.txt.
    """
    if file_path is None:
        current_dir = dirname(realpath(__file__))
        file_path = join(current_dir, 'unwanted_matches.txt')
    with open(file_path) as f:
        unwanted_matches = [m for m in f.read().splitlines()]
    return [q for q in marks if q not in unwanted_matches]


def trim_mark(mark):
    mark = mark.strip()
    mark = re.sub('\s+', ' ', mark)
    return mark


def get_metadata(metadata_num):
    return 'txt_{:03d}'.format(metadata_num + 1)


def assign_metadatas(steps):
    i = 0
    for step in steps:
        for control in step.controls:
            # <<line>> and <<text>> do not consume metadata
            if not isinstance(control, LabelControl):
                control.metadata_name = get_metadata(i)
                i += 1


def get_steps(marks):
    """
    Parses a list of marks and extracts the step-control hierarchy. It also
    links the metadatas to the controls.
    """
    # Trim spaces
    marks = [trim_mark(q) for q in marks]
    marks = remove_unwanted_matches(marks)

    steps = [Step(DEFAULT_STEP_NAME, DEFAULT_STEP_GROUP_NAME)]

    for mark in marks:
        # If it's a step mark:
        step_name = parse_mark(mark, 'step')
        if step_name is not None:
            steps += [Step(step_name, DEFAULT_STEP_GROUP_NAME)]
        else:
            # If it's not a step mark,it must be a control
            steps[-1].add_control(make_control(mark))

    # Remove duplicate controls
    for step in steps:
        step.remove_duplicate_controls()

    # Assign metadatas
    assign_metadatas(steps)

    return steps




class Step(object):
    def __init__(self, name, group_name):
        self.name = name
        self.group_name = group_name
        self.controls = []

    def add_control(self, control):
        self.controls += [control]

    def remove_duplicate_controls(self):
        """Remove duplicate controls, except line controls"""
        unique_controls = []
        for control in self.controls:
            for unique_control in unique_controls:
                if control == unique_control and \
                        (isinstance(control, ListControl) or
                         isinstance(control, StringControl)):
                    break
            else:
                unique_controls += [control]
        self.controls = unique_controls


class Control(object):
    def __init__(self, type, mark):
        self.type = type
        self.original_mark = mark
        self.metadata_name = None
        self.question = ""
        self.required = False
        self.default_value = None
        self.question_hidden = False

    def __eq__(self, other):
        return (self.original_mark, self.metadata_name, self.question) == \
            (other.original_mark, other.metadata_name, other.question)


class ListControl(Control):
    def __init__(self, mark):
        super().__init__('list', mark)
        parts = [part.strip() for part in mark.split(';')]
        self.question = parts[0]
        self.values = parts[1:]
        self.required = True

    def __eq__(self, other):
        return super().__eq__(other) and \
            isinstance(other, ListControl) and \
            self.values == other.values


class CheckboxControl(Control):
    def __init__(self, mark):
        super().__init__('checkbox', mark)
        self.type = 'checkbox'
        stripped = [part.strip() for part in mark.split(';')]
        self.question, self.label = stripped if len(stripped) == 2 \
            else ('', stripped[0])
        self._default_value = None

    def __eq__(self, other):
        return super().__eq__(other) and \
            isinstance(other, CheckboxControl)

    @property
    def default_value(self):
        """Wrapper to make sure that the default_value is a boolean"""
        return self._default_value

    @default_value.setter
    def default_value(self, value):
        if value is None:
            self._default_value = None
        elif type(value) is bool:
            self._default_value = value
        else:
            value = value.lower()
            if value in ['on', 'true', 'ja', 'yes', 'selected']:
                self._default_value = True
            elif value in ['off', 'false', 'nee', 'no', 'unselected',
                           'not selected']:
                self._default_value = False
            else:
                # TODO: Throw an exception?
                pass


class StringControl(Control):
    def __init__(self, mark):
        super().__init__('string', mark)
        self.question = mark.strip()

    def __eq__(self, other):
        return super().__eq__(other) and \
            isinstance(other, StringControl)


class LabelControl(Control):
    def __init__(self, mark):
        super().__init__('label', mark)
        self.question = ''
        self.default_value = mark.strip()
        self.question_hidden = True

    def __eq__(self, other):
        return super().__eq__(other) and \
            isinstance(other, LabelControl)


class LineControl(LabelControl):
    def __init__(self, mark):
        super().__init__(mark)
        # Override type
        self.type = 'line'
        self.question = ''
        self.default_value = '_'*300

    def __eq__(self, other):
        return super().__eq__(other) and \
            isinstance(other, LineControl)


def make_control(mark):
    """
    Factory method to create different kinds of controls
    """
    # Exctract default value from the mark:
    eq = mark.find('=')
    default_value = None
    if eq >= 0:
        default_value = mark[eq+1:].strip()
        mark = mark[:eq]

    # Divide the mark in words
    words = mark.split()
    types = {
        'list': ListControl,
        'checkbox': CheckboxControl,
        'line': LineControl,
        'string': StringControl,
        'text': LabelControl
    }
    modifiers = ['required',
                 'optional',
                 'empty'
                 ]
    found_modifiers = []

    # The first element contains the type (or if it's not the type, then it's a
    # string
    if len(words) > 0 and words[0] in types.keys():
        type, i = (words[0], 1)
    else:
        type, i = ('string', 0)

    # Parse modifiers
    while i < len(words) and words[i] in modifiers:
        found_modifiers += [words[i]]
        i += 1

    # Create the control:
    control = types[type](' '.join(words[i:]))

    # Apply modifiers to the control
    if 'required' in found_modifiers:
        control.required = True
    elif 'optional' in found_modifiers:
        control.required = False

    if 'empty' in found_modifiers:
        control.question_hidden = True

    # Set default value (if specified)
    if default_value is not None:
        control.default_value = default_value

    return control



class Config:
    def __init__(self):
        self.defaultonderwerptekst = ""
        self.defaultondertekenaar = ""
        self.defaultfunctieondertekenaar = ""
        self.defaultbijlageuploaden = ""
        self.defaultmedischecategorie = "Medisch Declaratie"

    def get_value(self, key, string):
        """
        Gets the values for a key (if the strings contains that key at all)
        Args:
            key(str): The string for the key. E.g. 'defaultonderwerptekst'
            string(str): The string. E.g. 'defaultbijlageuploaden ja'
        Returns:
            The value (E.g. 'ja') if the string contains the key, or None
            otherwise
        """
        if string.startswith(key):
            if len(string) > len(key):
                return string[len(key) + 1:]
            else:
                return ''
        return None

    def parse_defaults(self, attributes):
        parsed_attributes = []
        for attr in attributes:
            onderwerptekst = self.get_value('defaultonderwerptekst', attr)
            ondertekenaar = self.get_value('defaultondertekenaar', attr)
            functieondertekenaar = self.get_value('defaultfunctieondertekenaar',
                                                  attr)
            bijlageuploaden = self.get_value('defaultbijlageuploaden', attr)
            medischecategorie = self.get_value('defaultmedischecategorie', attr)

            if onderwerptekst is not None:
                self.defaultonderwerptekst = onderwerptekst
                parsed_attributes += [attr]

            if ondertekenaar is not None:
                self.defaultondertekenaar = ondertekenaar
                parsed_attributes += [attr]

            if functieondertekenaar is not None:
                fo = functieondertekenaar
                self.defaultfunctieondertekenaar = fo[0].upper() + fo
                parsed_attributes += [attr]

            if bijlageuploaden is not None:
                self.defaultbijlageuploaden = bijlageuploaden.lower() == 'ja'
                parsed_attributes += [attr]

            if medischecategorie is not None:
                mc = medischecategorie
                self.defaultmedischecategorie = mc[0].upper() + mc
                parsed_attributes += [attr]

        return [attr for attr in attributes if attr not in parsed_attributes]





def parse_mark(mark, tag, separator=None):
    """
    Returns a Step representing the mark, or null if it's not a Step mark.
    Args:
        mark (str): The mark to parse, e.g. 'step myStep'
        tag (str): The expected begining of th emark, e.g. 'step'
        separator (str): The separator, e.g. ';', or None if there is none
    Returns:
        The rest of the mark stripped, or null if the tag didn't match
    """
    if mark.startswith(tag + ' '):
        mark = mark[len(tag):]
        if separator is None:
            return mark.strip()
        else:
            parts = mark.split(separator)
            return [part.strip() for part in parts]
    else:
        # If it does not match, return none
        return None



from setuptools import setup
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

version = {}
with open("word2wiz/version.py") as fp:
    exec(fp.read(), version)

setup(
    name='word2wiz',

    version=version['__version__'],

    description='A tool to create wizard configurations from MS word documents',
    long_description=long_description,

    url='http://github.com/joserc87/word2wiz',

    author='Jose Ramon Cano',
    author_email='jose.cano@theconsultancyfirm.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',

        'Intended Audience :: Developers',
        'Topic :: Utilities',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    keywords='word2wiz document-wizard docwiz spell',

    packages=['word2wiz'],

    install_requires=['Flask', 'Jinja2', 'python-docx'],

    extras_require={
        'test': ['behave'],
    },

    package_data={
        'word2wiz': ['*.txt', 'spell/*.spl']
    },

    entry_points={
        'console_scripts': [
            'word2wiz=word2wiz:main',
        ],
    },
)



#!/usr/bin/python
"""
Reads a word document (docx format) and generates a spell based on the markers
found in the document. The spell is printed on the standard output by default
"""

import argparse
from os.path import basename, splitext, dirname, realpath, join

from jinja2 import Environment, FileSystemLoader

from .config import Config
from . import word
from . import mark_parser
from .spell_helper import LineControl, ListControl, CheckboxControl


def get_field_txt(control):
    if isinstance(control, LineControl):
        return ''
    elif isinstance(control, CheckboxControl):
        return control.question or control.label or ''
    else:
        return control.question or control.default_value or ''

def get_field_options(control):
    if isinstance(control, ListControl):
        return ' | '.join(control.values)
    elif isinstance(control, CheckboxControl):
        return ' | '.join(['True', 'False'])
    return ''


def tabularize(columns, rows):
    """
    Formats data like a table
    """

    # Find the width of the columns
    col_lens = [l for l in map(len, columns)]
    for row in rows:
        if isinstance(row, list):
            col_lens = [max(col_lens[i], len(row[i])) for i in range(len(row))]

    # Print heading
    divider = '+-{0}-+'.format('-+-'.join(['-'*l for l in col_lens]))
    semidivider = '| {0} +-{1}-+'.format(
        ' '*col_lens[0],
        '-+-'.join(['-'*l for l in col_lens[1:]]))
    columns = [name.center(width) for name, width in zip(columns, col_lens)]
    heading = '| {0} |'.format(' | '.join(columns))

    # Print data for row in
    report = [divider] + [heading] + [divider]
    for row in rows:
        if row == 'divider':
            report += [divider]
        elif row == 'semidivider':
            report += [semidivider]
        else:
            row = [name.ljust(width) for name, width in zip(row, col_lens)]
            report += ['| {0} |'.format(' | '.join(row))]

    # Separate the lines of the report by line feeds.
    return '\r\n'.join(report) + '\r\n'


def generate_report(steps):
    data = []

    col_names = ['STEP',
                 'FIELD',
                 'METADATA',
                 'OPTIONS']

    for step in steps:
        row = []
        step_name = step.name or ''
        for control in step.controls:
            field = get_field_txt(control)
            metadata = control.metadata_name or ''
            options = get_field_options(control)

            if isinstance(control, LineControl):
                row = 'semidivider'
            else:
                row = [step_name,
                       field,
                       metadata,
                       options]

            data += [row]
            # Only show the step name on the first row
            step_name = ''
        data += ['divider']

    return tabularize(col_names, data)


def word2wiz(path):
    # Jinja2
    current_dir = dirname(realpath(__file__))
    spell_dir = join(current_dir, 'spell')
    env = Environment(loader=FileSystemLoader(spell_dir),
                      trim_blocks=True,
                      lstrip_blocks=True)
    main_template = env.get_template('main.spl')

    # Get all the <<mark>>s from the word document
    marks = word.analyse_doc(path)

    # Take out the marks that are just for the configuration part
    config = Config()
    marks = config.parse_defaults(marks)

    # The steps will have the name of the document
    mark_parser.DEFAULT_STEP_NAME = splitext(basename(path))[0]
    # Parse the marks and get the step-control hierarchy
    steps = mark_parser.get_steps(marks)

    # Medewerkers for step 1 (name, last name, function)
    medewerkers = [
        ('Margot',    'Smits',       'Senior medisch adviseur'),
        ('Wendy',     'Haanschoten', 'Adviserend apotheker'),
        ('Marjolein', 'Rijkeboer',   'Adviserend psycholoog'),
        ('Herman',    'Flens',       'Medisch adviseur'),
        ('Job',       'van Huizen',  'Medisch adviseur')]
    medischecategorie = [
        'Medisch Polis',
        'Medisch Machtiging',
        'Medisch Declaratie',
        'Medisch Declaratie Dossier',
        'Medisch Zorgbemiddel',
        'Medisch Verhaal',
        'Medisch AWBZ',
        'Medisch Arbo',
        'Polis',
        'Standaard',
        'Marketing',
        'Financieel']
    spell = main_template.render(doc_name=splitext(basename(path))[0],
                                 config=config,
                                 medewerkers=medewerkers,
                                 medischecategorie=medischecategorie,
                                 steps=steps)
    report = generate_report(steps)

    return (spell, report)


def main():
    description = 'Create a spell configuration from a Microsoft Word document'
    parser = argparse.ArgumentParser(description=description)

    parser.add_argument('input_file',
                        nargs=1,
                        metavar='FILE',
                        help='the input word document')

    parser.add_argument('-o', '--output',
                        help='the file for the output spell (stdout by default)'
                        )

    parser.add_argument('--report',
                        help='the ouput report file containing the fields and' +
                        'the metadatas they are linked to')
    # Parse
    args = parser.parse_args()

    # Run
    spell, report = word2wiz(args.input_file[0])

    if args.output:
        # Write the spell
        with open(args.output, 'w') as output_file:
            output_file.write(spell)
    else:
        # By default, print the spell on the stdout
        print(spell)

    if args.report:
        # Write the report
        with open(args.report, 'w') as report_file:
            report_file.write(report)
    elif args.output:
        # Only print the report if the spell hasn't been printed
        print(report)


if __name__ == "__main__":
    main()
